dofile_once("data/scripts/lib/utilities.lua")

local w = 70
local h = 48
BiomeMapSetSize(w, h)
BiomeMapLoadImage(0, 0, "data/biome_impl/biome_map.png")

SetRandomSeed(4573, 4621)
SetRandomSeed(Random(1000, 9999), Random(1000, 9999))

local biomeList = { 
	{
		--Coal Mines (Mines)
		color = 0xffd57917
	},
	{
		--Coal Mines Alt (Collapsed Mines)
		color = 0xffd56517
	},
	{
		--Town Under (Twisty Passages)
		color = 0xff6ba04b
	},
	{
		--Excavation Site (Coal Pits)
		color = 0xff124445
	},
	{
		--Fungi Cave (Fungal Caverns)
		color = 0xffe861f0
	},
	{
		--Snow Caves (Snowy Depths)
		color = 0xff1775d5
	},
	{
		--Snow Castle (Hiisi Base)
		color = 0xff0046ff
	},
	{
		--Rainforest Open (Underground Jungle)
		color = 0xffa08400
	},
	{
		--Rainforest (Underground Jungle 2)
		color = 0xff808000
	},
	{
		--Vault
		color = 0xff008000
	},
	{
		--Sand Caves
		color = 0xffe1cd32
	},
	{
		--Frozen Vault
		color = 0xff0080a8
	},
	{
		--Wand Cave (Magical Temple)
		color = 0xff006c42
	},
	{
		--The End (Hell)
		color = 0xff3c0f0a
	},
	{
		--Crypt (Temple of the Art)
		color = 0xff786c42
	},
	{
		--Pyramid
		color = 0xff967f11
	},
	{
		--Clouds
		color = 0xff36d5c9
	}
}

--Temp fix for BiomeMapGetPixel returning incorrect byte value, provided by Dregu from the Noita Discord.
local function intToByte(n)
	n = (n < 0) and (4294967296 + n) or n
	return (math.modf(n/16777216))%256, (math.modf(n/65536))%256, (math.modf(n/256))%256, n%256
end
local function byteToInt(b1, b2, b3, b4)
	local n = b1*16777216 + b2*65536 + b3*256 + b4
	n = (n > 2147483647) and (n - 4294967296) or n
	return n
end

RealBiomeMapGetPixel = BiomeMapGetPixel

local function BiomeMapGetPixel(x, y)
  local wrong = RealBiomeMapGetPixel(x, y)
  local a,r,g,b = intToByte(wrong)
  local right = byteToInt(a,b,g,r)+2^32
  return right
end

local pixels = { }
--Get all the pixels.
for x=0, w-1 do
	for y=0, h-1 do
		--Get the pixel for this co-ordinate.
		local thisPixel = BiomeMapGetPixel(x, y)
		--print("Pixel (" .. x .. "," .. y .. ") = " .. thisPixel)
		--Add the pixel to the pixels list if it matches a biome from the biome list.
		for i, biome in ipairs(biomeList) do
			if thisPixel == biome.color then
				--print("Got Pixel (" .. x .. "," .. y .. ") : " .. thisPixel)
				table.insert(pixels, {
					px = x,
					py = y,
					color = biome.color
				})
			end
		end
	end
end

--TODO: Fix
--This func takes aTable and shuffles it.
local function shuffle(aTable)
	local ret = { }
	for i, entry in ipairs(aTable) do
		table.insert(ret, entry)
	end
    for i=1, #ret-1 do
        local j = Random(i, #ret)
        ret[i], ret[j] = ret[j], ret[i]
    end
	return ret
end
--Shuffle the biome list to a new table.
local newBiomeList = shuffle(biomeList)
--Check the list was shuffled.
for i, biome in ipairs(newBiomeList) do
	print("New: " .. biome.color .. " Old: " .. biomeList[i].color)
end

--for each pixel we check through the original biome list.
for i, pixel in ipairs(pixels) do
	for j, biome in ipairs(biomeList) do
		--If the biome matches then we set the pixel to the same index but from the shuffled table.
		if pixel.color == biome.color then
			BiomeMapSetPixel(pixel.px, pixel.py, newBiomeList[j].color)
		end
	end
end